//
//			Y. Okamoto, 2009.11
//
#include "stdafx.h"
#include "rnkiss.h"


namespace MyRNKISS {

	myrn::myrn() {
		i = 0x95555555;
		j = 0x56666666;
		k = 0x39999999;
	}

	myrn::myrn( unsigned __int32 iv, unsigned __int32 jv, unsigned __int32 kv ){
		i = iv;
		j = jv;
		k = kv;
	}


	void myrn::init(){
		i = 0x95555555;
		j = 0x56666666;
		k = 0x39999999;
	}


	void myrn::init( unsigned __int32 iv, unsigned __int32 jv, unsigned __int32 kv ){
		i = iv;
		j = jv;
		k = kv;
	}


	//
	//     cf. Robert and Casella (1999) "Monte Carlo Statistical Methods", 
	//                                    Algorithm A.3
	//
	//                     0.0 <  uni()  <  1.0      
	//
	double myrn::uni() {
       const double den = ((double)0xFFFFFFFF) + 2.0;

		i = 69069 * i + 23606797;
		j ^= j << 17;
		j ^= j >> 15;
		k = (k ^ (k << 18)) & 0x7FFFFFFF;
		k ^= k >> 13;

		unsigned __int32 v = i + j + k;

		return (((double)v) + 1.0) / den;
	}


    //        Standard Normal Distribution
	double myrn::normal(){
		double v1, v2, w;
		do {
    		v1 = 2.0 * uni() - 1.0;
			v2 = 2.0 * uni() - 1.0;
			w  = sqr(v1) + sqr(v2);
		} while (((w >= 1.0) || (w <= 0.0)));

		double c = Math::Sqrt(-2.0 * Math::Log(w)/w);

		return c * v1;
	}


    //    Normal Distribution of mean m  and standard deviation s
	double myrn::normalMS(double m, double s){
			double v = s * normal() + m;
			return  v;
	}


    //    A Pair of Independent Random Numbers of 
	//      the Standard Normal Distribution 
	void  myrn::normalPair( double & n1, double & n2 ){
			double v1, v2, w, c;
			do {
				  v1 = 2.0 * uni() - 1.0;
				  v2 = 2.0 * uni() - 1.0;
				  w  = sqr(v1) + sqr(v2);
			} while (((w >= 1.0) || (w <= 0.0)));

			c = Math::Sqrt(-2.0 * Math::Log(w) / w);

			n1 = c * v1;
			n2 = c * v2;
	}

}
